//-----------------------------------------------------------------------------
/*
 * Description: This file contains the class Ax::Mail::MailBoxMgr
 *
 * See the file README in the main directory for a description of
 * this software, copyright information, and how to reach the author.
 *
 * Author: alex
 * Date:   15.11.2003
 *
 * Last modfied:
 *   $Author: alex $
 *   $Date: 2003/12/26 19:26:54 $
 *   $Revision: 1.9 $
 */
//-----------------------------------------------------------------------------
#ifndef __Ax_Mail_MailBoxMgr_H__
#define __Ax_Mail_MailBoxMgr_H__

//-----------------------------------------------------------------------------
//     includes
//-----------------------------------------------------------------------------

//----- qt --------------------------------------------------------------------
//----- CORBA -----------------------------------------------------------------
//----- ICE -------------------------------------------------------------------
//----- C++ -------------------------------------------------------------------
//----- C ---------------------------------------------------------------------
//----- AxLib -----------------------------------------------------------------
#include <Ax/Mail/Globals.h>
#include <Ax/Mail/Export.h>

//----- local -----------------------------------------------------------------
//----- vdr -------------------------------------------------------------------
//----- c-client --------------------------------------------------------------
//----- other includes --------------------------------------------------------

//=============================================================================
//     namespace Ax
//=============================================================================
namespace Ax
{
  //===========================================================================
  //     namespace Mail
  //===========================================================================
  namespace Mail
  {
    //=============================================================================
    //     class MailBoxMgr
    //=============================================================================
    /** The singleton MailBoxMgr is responsible for all instances of MailBox.
     *
     *  The main purpose of this class is to demultiplex answers of \c c-client /
     *  from the mail-server and to forward them to the associated instance
     *  of MailBox.
     *
     *  \todo overwork the implementation of this class, especially the
     *        handling of open mailboxes and the handling of mailbox-instances.
     */
    class AX_MAIL_EXPORT MailBoxMgr
    {
        //-------------------------------------------------------------------------
        //     friends
        //-------------------------------------------------------------------------
        /** MailBox is friend of this to be allowed to call registerMailBox() and
         *  deregisterMailBox()
         */
        friend class MailBox;

      private:
        //-------------------------------------------------------------------------
        //     MailBoxMgr()
        //-------------------------------------------------------------------------
        /** Constructor
        */
        MailBoxMgr();

        //-------------------------------------------------------------------------
        //     ~MailBoxMgr()
        //-------------------------------------------------------------------------
        /** Destructor
         */
        ~MailBoxMgr();
        
        //-------------------------------------------------------------------------
        //     registerMailBox()
        //-------------------------------------------------------------------------
        /** An instance of MailBox registers itself in the constructor.
         */
        bool registerMailBox(MailBox *theMailBox);

        //-------------------------------------------------------------------------
        //     deregisterMailBox()
        //-------------------------------------------------------------------------
        /** An instance of MailBox registers itself in the destructor.
         */
        void deregisterMailBox(MailBox *theMailBox);

        //-------------------------------------------------------------------------
        //     registerOpenMailBox()
        //-------------------------------------------------------------------------
        /** An instance of MailBox registers itself when opened.
         */
        bool registerOpenMailBox(MailBox *theMailBox);

        //-------------------------------------------------------------------------
        //     deregisterOpenMailBox()
        //-------------------------------------------------------------------------
        /** An instance of MailBox registers itself when closed.
         */
        void deregisterOpenMailBox(MailBox *theMailBox);

      public:
        //-------------------------------------------------------------------------
        //     instance()
        //-------------------------------------------------------------------------
        /** Returns the only instance of this singleton.
         *
         */
        static MailBoxMgr *instance();

        //-------------------------------------------------------------------------
        //     terminate()
        //-------------------------------------------------------------------------
        /** cleanup remaining MailBox-instances and destroy myself.
         *
         *  this method must be called at the end of the application.
         */
        static void terminate();

        //-------------------------------------------------------------------------
        //     destroyAll()
        //-------------------------------------------------------------------------
        /** cleanup all registered MailBox-instances.
         */
        void destroyAll();

        //-------------------------------------------------------------------------
        //     setDebugCClientCallbacks()
        //-------------------------------------------------------------------------
        void setDebugCClientCallbacks(bool fDebug);

        //-------------------------------------------------------------------------
        //     getMailBox()
        //-------------------------------------------------------------------------
        /** finds the mailbox for a given stream
         *
         *  if no mailbox for the given stream could be found and
         *  an active mailbox is set, the active mailbox will be returned.
         */
        MailBox *getMailBox(MAILSTREAM *theStream = 0);

        //-------------------------------------------------------------------------
        //     getMailBoxCltn()
        //-------------------------------------------------------------------------
        /** Returns the internal collection of MailBox-instances
         */
        const MailBoxList &getMailBoxCltn() const;

        //-------------------------------------------------------------------------
        //     setPendingMailBox()
        //-------------------------------------------------------------------------
        /** sets the mailbox which initiated the last command to the
         *  \ref c_client -library.
         */
        void setActiveMailBox(MailBox *theMailBox = 0) { myActiveMailBox = theMailBox; }

        //-------------------------------------------------------------------------
        //     getActiveMailBox()
        //-------------------------------------------------------------------------
        /** returns the active mailbox which expects the next answers if no
         *  mailbox with an associated stream could be located.
         */
        MailBox *getActiveMailBox() const { return myActiveMailBox; }

        //-------------------------------------------------------------------------
        //     hasMailBoxPtr()
        //-------------------------------------------------------------------------
        /** checks if the given mailbox-pointer is registered
         */
        bool hasMailBoxPtr(MailBox *theMailBox) const;

      private:
        //-------------------------------------------------------------------------
        //     attributes
        //-------------------------------------------------------------------------
        static MailBoxMgr *myInstance;

        MailBox           *myActiveMailBox;
        Stream2MailBoxMap  myOpenMailBoxes;
        MailBoxList        myMailBoxes    ;

        static bool        myIsCClientInitialized;

    }; // class MailBoxMgr

  }; // namespace Mail

}; // namespace Ax

#endif

