//-----------------------------------------------------------------------------
/*
 * Description: This file contains the class Ax::Mail::MailBoxSetup
 *
 * See the file README in the main directory for a description of
 * this software, copyright information, and how to reach the author.
 *
 * Author: alex
 * Date:   15.11.2003
 *
 * Last modfied:
 *   $Author: alex $
 *   $Date: 2004/04/04 15:44:23 $
 *   $Revision: 1.8 $
 */
//-----------------------------------------------------------------------------
#ifndef __Ax_Mail_MailBoxSetup_H__
#define __Ax_Mail_MailBoxSetup_H__

//-----------------------------------------------------------------------------
//     includes
//-----------------------------------------------------------------------------

//----- qt --------------------------------------------------------------------
//----- CORBA -----------------------------------------------------------------
//----- ICE -------------------------------------------------------------------
//----- C++ -------------------------------------------------------------------
//----- C ---------------------------------------------------------------------
//----- AxLib -----------------------------------------------------------------
#include <Ax/Mail/Globals.h>
#include <Ax/Mail/Export.h>

//----- local -----------------------------------------------------------------
//----- vdr -------------------------------------------------------------------
//----- c-client --------------------------------------------------------------
//----- other includes --------------------------------------------------------

//=============================================================================
//     namespace Ax
//=============================================================================
namespace Ax
{
  //===========================================================================
  //     namespace Mail
  //===========================================================================
  namespace Mail
  {
    //=========================================================================
    //     class MailBoxSetup
    //=========================================================================
    /** Small class to hold the settings for one mailbox.
    */
    class AX_MAIL_EXPORT MailBoxSetup
    {
      public:
        //-------------------------------------------------------------------------
        //     MailBoxSetup()
        //-------------------------------------------------------------------------
        MailBoxSetup();

        //-------------------------------------------------------------------------
        //     MailBoxSetup()
        //-------------------------------------------------------------------------
        MailBoxSetup(const MailBoxSetup &that);

        //-------------------------------------------------------------------------
        //     operator=()
        //-------------------------------------------------------------------------
        MailBoxSetup &operator=(const MailBoxSetup &that);

        //-------------------------------------------------------------------------
        //     clear()
        //-------------------------------------------------------------------------
        void clear();

        //-------------------------------------------------------------------------
        //     enums
        //-------------------------------------------------------------------------
        // The following two enums should be in application-specific
        // code and not here! But for simplicity I add them here...

        /** Backgound check mode
         */
        enum eBGCheckMode
        {
          BG_EXT_OFF  = 0x00   ///< don't start external app
        , BG_EXT_ON   = 0x01   ///< do    start external app
        , BG_EXT_MASK = 0x01   ///< bit-mask for BG_EXT_*

        , BG_MSG_OFF  = 0x00   ///< don't display a message
        , BG_MSG_ON   = 0x02   ///< do    display a message
        , BG_MSG_MASK = 0x02   ///< bit-mask for BG_MSG_*


        , BG_MODE_NONE     = BG_EXT_OFF | BG_MSG_OFF
        , BG_MODE_EXT      = BG_EXT_ON  | BG_MSG_OFF
        , BG_MODE_MSG      = BG_EXT_OFF | BG_MSG_ON
        , BG_MODE_EXT_MSG  = BG_EXT_ON  | BG_MSG_ON

        , BG_MODE_FIRST    = BG_MODE_NONE
        , BG_MODE_LAST     = BG_MODE_EXT_MSG
        }; // enum MailViewMode

        /** Mailbox-access mode
         */
        enum eAccessMode
        {
          AM_NONE       = 0x00   ///< no locks
        , AM_SETUP      = 0x01   ///< code required for setup
        , AM_SETUP_READ = 0x02   ///< code required for setup and mail-read access

        , AM_FIRST      = AM_NONE
        , AM_LAST       = AM_SETUP_READ
        }; // enum MailViewMode

        //-------------------------------------------------------------------------
        //     attributes
        //-------------------------------------------------------------------------
        char        AccountName[ 80];
        char        LoginName  [ 80];
        char        LoginPass  [ 80];
        char        MailBox    [120];

        int         fAutoMarkSeen   ; ///< mark mails as seen when viewed
        int         fExpungeOnClose ; ///< expunge (delete marked mails) on close?

        // The following attributes should be in an application-specific
        // class and not here. But for simplicity I add them here...

        char        AccessCode [ 10]; ///< code to access the mailbox
        int         AccessMode      ; ///< Access mode: AM_*
        int         BGCheckMode     ; ///< background check mode: BG_*

    }; // class MailBoxSetup

  }; // namespace Mail

}; // namespace Ax

#endif

