//-----------------------------------------------------------------------------
/*
 * Description: This file contains the class Ax::Mail::Mail
 *
 * See the file README in the main directory for a description of
 * this software, copyright information, and how to reach the author.
 *
 * Author: alex
 * Date:   15.11.2003
 *
 * Last modfied:
 *   $Author: alex $
 *   $Date: 2003/12/19 11:05:53 $
 *   $Revision: 1.11 $
 */
//-----------------------------------------------------------------------------

//=============================================================================
//     includes
//=============================================================================
//----- qt --------------------------------------------------------------------
//----- CORBA -----------------------------------------------------------------
//----- ICE -------------------------------------------------------------------
//----- C++ -------------------------------------------------------------------
#include <algorithm>

//----- C ---------------------------------------------------------------------
//----- AxLib -----------------------------------------------------------------
#include <Ax/Tools/Trace.h>
// #include <Ax/Tools/TraceOn.h>
#include <Ax/Mail/Mail.h>
#include <Ax/Mail/MailPart.h>
#include <Ax/Mail/MailBox.h>
#include <Ax/Mail/MailBoxMgr.h>

//----- vdr -------------------------------------------------------------------
//----- local plugin ----------------------------------------------------------
//----- locals ----------------------------------------------------------------

//=============================================================================
//     defines
//=============================================================================

//=============================================================================
//     using namespaces
//=============================================================================
using namespace std;
using namespace Ax::Mail;

//=============================================================================
//     class Mail
//=============================================================================

//-------------------------------------------------------------------------
//     Mail::Mail()
//-------------------------------------------------------------------------
Mail::Mail(MailBox *theMailBox, MESSAGECACHE *theMsgCache)
      : myMailBox    (theMailBox)
      , myRootPart   (0)
      , myMsgCache   (theMsgCache)
{
//  wsdebug(("> Mail::Mail() MailNum: %ld", myMailNum));
//  wsdebug(("< Mail::Mail() MailNum: %ld", myMailNum));
} // Mail::Mail()

//-------------------------------------------------------------------------
//     Mail::~Mail()
//-------------------------------------------------------------------------
Mail::~Mail()
{
//  wsdebug(("> Mail::~Mail() MailNum: %ld", myMailNum));
//  wsdebug(("< Mail::~Mail() MailNum: %ld", myMailNum));
} // Mail::~Mail()

//-------------------------------------------------------------------------
//     Mail::getMailBox()
//-------------------------------------------------------------------------
MailBox *Mail::getMailBox() const
{
  return myMailBox;
} // Mail::getMailBox()

//-------------------------------------------------------------------------
//     Mail::getRootPart()
//-------------------------------------------------------------------------
MailPart *Mail::getRootPart()
{
  if (myRootPart == 0)
  {
    wsinfo(("retrieving mail-structure %ld from mailbox '%s'...", getMailNum(), getMailBox()->getMailBox()));

    wsdebug(("> Mail::getRootPart() MailNum: %ld", getMailNum()));

    BODY *aMsgBody;

    MailBoxMgr::instance()->setActiveMailBox(myMailBox);

    mail_fetchstructure(myMailBox->stream(), getMailNum(), &aMsgBody);

    MailBoxMgr::instance()->setActiveMailBox(0);

    if (aMsgBody == 0)
    {
      wserror(("retrieving mail-structure %ld from mailbox '%s'...FAILED"
            , getMailNum(), getMailBox()->getMailBox()
            ));
    }
    else
    {
      myRootPart = new MailPart( this
                                 , 0        // no parent
                                 , aMsgBody // BODY-structure
                                 , (aMsgBody->type == TYPEMULTIPART) ? 0 : 1
                                 );

      wsinfo(("retrieving mail-structure %ld from mailbox '%s'...OK"
            , getMailNum(), getMailBox()->getMailBox()
            ));
    } // if
    wsdebug(("< Mail::getRootPart() MailNum: %ld", getMailNum()));
  } // if
  return myRootPart;
} // Mail::getRootPart()

//-------------------------------------------------------------------------
//     Mail::getMailNum()
//-------------------------------------------------------------------------
long Mail::getMailNum() const
{
  return myMsgCache ? myMsgCache->msgno : 0;
} // Mail::getMailNum()

//-------------------------------------------------------------------------
//     Mail::getIsRecent()
//-------------------------------------------------------------------------
bool Mail::getIsRecent() const
{
  return myMsgCache ? myMsgCache->recent : false;
} // Mail::getIsRecent()

//-------------------------------------------------------------------------
//     Mail::getIsSeen()
//-------------------------------------------------------------------------
bool Mail::getIsSeen() const
{
  return myMsgCache ? myMsgCache->seen : false;
} // Mail::getIsSeen()

//-------------------------------------------------------------------------
//     Mail::getIsFlagged()
//-------------------------------------------------------------------------
bool Mail::getIsFlagged() const
{
  return myMsgCache ? myMsgCache->flagged : false;
} // Mail::getIsFlagged()

//-------------------------------------------------------------------------
//     Mail::getIsAnswered()
//-------------------------------------------------------------------------
bool Mail::getIsAnswered() const
{
  return myMsgCache ? myMsgCache->answered : false;
} // Mail::getIsAnswered()

//-------------------------------------------------------------------------
//     Mail::getIsDeleted()
//-------------------------------------------------------------------------
bool Mail::getIsDeleted() const
{
  return myMsgCache ? myMsgCache->deleted : false;
} // Mail::getIsDeleted()

//-------------------------------------------------------------------------
//     Mail::getBodySize()
//-------------------------------------------------------------------------
long Mail::getBodySize() const
{
  return myMsgCache ? myMsgCache->rfc822_size: 0;
} // Mail::getBodySize()

//-------------------------------------------------------------------------
//     Mail::getFlatPartList()
//-------------------------------------------------------------------------
MailPartList Mail::getFlatPartList(bool fWithMultiPart)
{
  MailPartList aList;
  
  if (getRootPart())
  {
    getRootPart()->addTo(aList, fWithMultiPart);
  } // if

  return aList;
} // Mail::getFlatPartList()

//-------------------------------------------------------------------------
//     Mail::getUID()
//-------------------------------------------------------------------------
unsigned long Mail::getUID() const
{
  if (myMsgCache == 0)
  {
    return 0;
  } // if

  MailBoxMgr::instance()->setActiveMailBox(myMailBox);
  unsigned long anUID = mail_uid(myMailBox->stream(), getMailNum());
  MailBoxMgr::instance()->setActiveMailBox();

  return anUID;
} // Mail::getUID()

//-------------------------------------------------------------------------
//     Mail::getDate()
//-------------------------------------------------------------------------
std::string Mail::getDate() const
{
  MailBoxMgr::instance()->setActiveMailBox(myMailBox);
  ENVELOPE *anEnvelope = mail_fetchstructure(myMailBox->stream(), getMailNum(), NIL);
  MailBoxMgr::instance()->setActiveMailBox();

  string aStr;

  if (anEnvelope && anEnvelope->date)
  {
    aStr = string(anEnvelope->date);
  } // if
  else
  {
    wswarn(("unable to get date for mail %ld in mailbox '%s'"
          , getMailNum(), getMailBox()->getMailBox()
          ));
  } // if

  return aStr;
} // Mail::getDate()

//-------------------------------------------------------------------------
//     Mail::getFrom()
//-------------------------------------------------------------------------
std::string Mail::getFrom() const
{
  if (myMsgCache == 0)
  {
    return "";
  } // if

  char szText[128];

  MailBoxMgr::instance()->setActiveMailBox(myMailBox);
  mail_fetchfrom(szText, myMailBox->stream(), getMailNum(), sizeof(szText) - 1);
  MailBoxMgr::instance()->setActiveMailBox();

  // since c-client-libaray fills the string up to the length
  // with spaces we have to strip them away
  for ( char *p = szText + sizeof(szText) - 1;
        p > szText && (*p == ' ' || *p == '\0');
        p--)
  {
    *p = '\0';
  } // while

  string aStr((const char *)szText);

  return aStr;
} // Mail::getFrom()

//-------------------------------------------------------------------------
//     Mail::getSubject()
//-------------------------------------------------------------------------
std::string Mail::getSubject(int theMaxLen) const
{
  if (myMsgCache == 0)
  {
    return "";
  } // if

  const int aMaxLen = 4096;
  char szText[aMaxLen];
  int  aLen = (theMaxLen > 0 && theMaxLen < aMaxLen) ? theMaxLen : aMaxLen;

  MailBoxMgr::instance()->setActiveMailBox(myMailBox);
  mail_fetchsubject(szText, myMailBox->stream(), getMailNum(), aLen - 1);
  MailBoxMgr::instance()->setActiveMailBox();
  szText[aLen - 1] = '\0';

  string aStr((const char *)szText);

  return aStr;
} // Mail::getSubject()

//-------------------------------------------------------------------------
//     Mail::getFlagsStr()
//-------------------------------------------------------------------------
std::string Mail::getFlagsStr() const
{
  char szFlags[10];

  int i = 0;

//  szFlags[i++] =  getIsRecent  ()  ? 'R' : '_';
  szFlags[i++] =  getIsSeen    ()  ? '_' : 'N';
  szFlags[i++] =  getIsFlagged ()  ? 'F' : '_';
  szFlags[i++] =  getIsAnswered()  ? 'A' : '_';
  szFlags[i++] =  getIsDeleted ()  ? 'D' : '_';
  szFlags[i++] = '\0';

  return std::string(szFlags);
} // Mail::getFlagsStr()

//-------------------------------------------------------------------------
//     Mail::getBody()
//-------------------------------------------------------------------------
std::string Mail::getBody() const
{
  if (myMsgCache == 0)
  {
    return "";
  } // if

  unsigned long aLen   = 0;
  long          aFlags = (!myMailBox->getSetup()->fAutoMarkSeen) ? FT_PEEK : 0
                       ;
  MailBoxMgr::instance()->setActiveMailBox(myMailBox);
  char *aMailBody = mail_fetchtext_full( myMailBox->stream()
                                       , getMailNum()
                                       , &aLen
                                       , aFlags
                                       );
  MailBoxMgr::instance()->setActiveMailBox();

  string aStr((const char *)aMailBody);

  return aStr;
} // Mail::getBody()

//-------------------------------------------------------------------------
//     Mail::getFlagStr()
//-------------------------------------------------------------------------
std::string Mail::getFlagStr(MailFlag theFlag)
{
  switch (theFlag)
  {
    case Ax::Mail::MF_SEEN    : return "\\Seen"    ;
    case Ax::Mail::MF_FLAGGED : return "\\Flagged" ;
    case Ax::Mail::MF_DELETED : return "\\Deleted" ;
    case Ax::Mail::MF_ANSWERED: return "\\Answered";
  } // switch
  return "";
} // Mail::getFlagStr()

//-------------------------------------------------------------------------
//     Mail::setFlag()
//-------------------------------------------------------------------------
void Mail::setFlag(MailFlag theFlag)
{
  if (myMsgCache == 0)
  {
    return;
  } // if

  std::string aFlagStr = getFlagStr(theFlag);
  wsdebug(("> Mail::setFlag() Num: %3ld  %s", getMailNum(), aFlagStr.c_str()));

  char szMailNum[20];
  sprintf(szMailNum, "%ld", getMailNum());

  wsinfo(("setting flag %s for mail %ld in mailbox '%s'"
        , aFlagStr.c_str(), getMailNum(), getMailBox()->getMailBox()
        ));

  MailBoxMgr::instance()->setActiveMailBox(myMailBox);
  mail_setflag_full( myMailBox->stream()
                   , szMailNum
                   , const_cast<char *>(aFlagStr.c_str())
                   , 0
                   );
  MailBoxMgr::instance()->setActiveMailBox();

  wsdebug(("< Mail::setFlag() Num: %3ld  %s", getMailNum(), aFlagStr.c_str()));
} // Mail::setFlag()

//-------------------------------------------------------------------------
//     Mail::clearFlag()
//-------------------------------------------------------------------------
void Mail::clearFlag(MailFlag theFlag)
{
  if (myMsgCache == 0)
  {
    return;
  } // if

  std::string aFlagStr = getFlagStr(theFlag);
  wsdebug(("> Mail::clearFlag() Num: %3ld  %s", getMailNum(), aFlagStr.c_str()));

  char szMailNum[20];
  sprintf(szMailNum, "%ld", getMailNum());

  wsinfo(("clearing flag %s for mail %ld in mailbox '%s'"
        , aFlagStr.c_str(), getMailNum(), getMailBox()->getMailBox()
        ));

  MailBoxMgr::instance()->setActiveMailBox(myMailBox);
  mail_clearflag_full( myMailBox->stream()
                     , szMailNum
                     , const_cast<char *>(aFlagStr.c_str())
                     , 0
                     );
  MailBoxMgr::instance()->setActiveMailBox();

  wsdebug(("< Mail::clearFlag() Num: %3ld  %s", getMailNum(), aFlagStr.c_str()));
} // Mail::clearFlag()

//-------------------------------------------------------------------------
//     Mail::getFlag
//-------------------------------------------------------------------------
bool Mail::getFlag(MailFlag theFlag) const
{
  bool fSet = false;
  switch (theFlag)
  {
    case MF_SEEN    : fSet = getIsSeen    (); break;
    case MF_FLAGGED : fSet = getIsFlagged (); break;
    case MF_DELETED : fSet = getIsDeleted (); break;
    case MF_ANSWERED: fSet = getIsAnswered(); break;
  } // switch

  return fSet;
} // Mail::getFlag

