//-----------------------------------------------------------------------------
/*
 * Description: This file contains the class Ax::Mail::MailBoxMgr
 *
 * See the file README in the main directory for a description of
 * this software, copyright information, and how to reach the author.
 *
 * Author: alex
 * Date:   15.11.2003
 *
 * Last modfied:
 *   $Author: alex $
 *   $Date: 2003/12/26 19:26:57 $
 *   $Revision: 1.9 $
 */
//-----------------------------------------------------------------------------

//=============================================================================
//     includes
//=============================================================================
//----- qt --------------------------------------------------------------------
//----- CORBA -----------------------------------------------------------------
//----- ICE -------------------------------------------------------------------
//----- C++ -------------------------------------------------------------------
#include <algorithm>

//----- C ---------------------------------------------------------------------
//----- AxLib -----------------------------------------------------------------
#include <Ax/Tools/Trace.h>
// #include <Ax/Tools/TraceOn.h>
#include <Ax/Mail/MailBox.h>
#include <Ax/Mail/MailBoxMgr.h>

//----- vdr -------------------------------------------------------------------
//----- local plugin ----------------------------------------------------------

//----- locals ----------------------------------------------------------------

//=============================================================================
//     defines
//=============================================================================

//=============================================================================
//     using namespaces
//=============================================================================
using namespace std;
using namespace Ax::Mail;

//=============================================================================
//     externals
//=============================================================================
/** in cclient_callbacks.cpp there's a variable DebugCClientCallbacks
 *  (not very elegant)
 */
namespace Ax
{
  namespace Mail
  {
    extern bool DebugCClientCallbacks;
  }; // namespace Mail
}; // namespace Ax

//=============================================================================
//     class MailBoxMgr
//=============================================================================

//-------------------------------------------------------------------------
//     globals
//-------------------------------------------------------------------------
MailBoxMgr *MailBoxMgr::myInstance             = 0;
bool        MailBoxMgr::myIsCClientInitialized = false;


//-------------------------------------------------------------------------
//     MailBoxMgr::MailBoxMgr()
//-------------------------------------------------------------------------
MailBoxMgr::MailBoxMgr()
          : myActiveMailBox(0)
{
  wsdebug(("> MailBoxMgr::MailBoxMgr()"));

  if (!myIsCClientInitialized)
  {
    // required for c-client-library: register mailbox drivers
#   include "linkage.c"
    myIsCClientInitialized = true;
  } // if

  wsdebug(("< MailBoxMgr::MailBoxMgr()"));
} // MailBoxMgr::MailBoxMgr()

//-------------------------------------------------------------------------
//     MailBoxMgr::~MailBoxMgr()
//-------------------------------------------------------------------------
MailBoxMgr::~MailBoxMgr()
{
  wsdebug(("> MailBoxMgr::~MailBoxMgr()"));
  wsdebug(("< MailBoxMgr::~MailBoxMgr()"));
} // MailBoxMgr::~MailBoxMgr()

//-------------------------------------------------------------------------
//     MailBoxMgr::setDebugCClientCallbacks()
//-------------------------------------------------------------------------
void MailBoxMgr::setDebugCClientCallbacks(bool fDebug)
{
  DebugCClientCallbacks = fDebug;
} // MailBoxMgr::setDebugCClientCallbacks()

//-------------------------------------------------------------------------
//     MailBoxMgr::registerMailBox()
//-------------------------------------------------------------------------
bool MailBoxMgr::registerMailBox(MailBox *theMailBox)
{
  wsdebug(("> MailBoxMgr::registerMailBox() MailBox: %p", theMailBox));

  if (theMailBox != 0)
  {
    if (std::find(myMailBoxes.begin(), myMailBoxes.end(), theMailBox) == myMailBoxes.end())
    {
      myMailBoxes.push_back(theMailBox);
    } // if
  } // if

  wsdebug(("< MailBoxMgr::registerMailBox() MailBox: %p", theMailBox));
  return true;
} // MailBoxMgr::registerMailBox()

//-------------------------------------------------------------------------
//     MailBoxMgr::deregisterMailBox()
//-------------------------------------------------------------------------
void MailBoxMgr::deregisterMailBox(MailBox *theMailBox)
{
  wsdebug(("> MailBoxMgr::deregisterMailBox() MailBox: %p", theMailBox));

  if (theMailBox != 0)
  {
    myMailBoxes.remove(theMailBox);
  } // if

  wsdebug(("< MailBoxMgr::deregisterMailBox() MailBox: %p", theMailBox));
} // MailBoxMgr::deregisterMailBox()

//-------------------------------------------------------------------------
//     MailBoxMgr::registerOpenMailBox()
//-------------------------------------------------------------------------
bool MailBoxMgr::registerOpenMailBox(MailBox *theMailBox)
{
  wsdebug(("> MailBoxMgr::registerOpenMailBox() MailBox: %p  isOpen: %d"
         , theMailBox
         , theMailBox->isOpen()
         ));
  bool fOK = true;

  if (theMailBox->isOpen())
  {
    Stream2MailBoxMap::iterator anIter = myOpenMailBoxes.find(theMailBox->stream());

    if (anIter != myOpenMailBoxes.end())
    {
      if (anIter->second != theMailBox)
      {
        // another MailBox is registerOpened with the same stream
        // this should not happen!
        wserror(("MailBoxMgr::registerOpenMailBox() inconsistent data -> another MailBox registerOpened for the same stream!"));
      } // if
    }
    else
    {
      myOpenMailBoxes.insert(std::make_pair(theMailBox->stream(), theMailBox));
    } // if
  } // if

  wsdebug(("< MailBoxMgr::registerOpenMailBox() MailBox: %p stream: %p"
         , theMailBox
         , theMailBox->stream()
         ));

  return fOK;
} // MailBoxMgr::registerOpenMailBox()

//-------------------------------------------------------------------------
//     MailBoxMgr::deregisterOpenMailBox()
//-------------------------------------------------------------------------
void MailBoxMgr::deregisterOpenMailBox(MailBox *theMailBox)
{
  wsdebug(("> MailBoxMgr::deregisterOpenMailBox() MailBox: %p stream: %p"
         , theMailBox
         , theMailBox->stream()
         ));

  if (theMailBox->stream() != 0)
  {
    Stream2MailBoxMap::iterator anIter = myOpenMailBoxes.find(theMailBox->stream());

    if (anIter != myOpenMailBoxes.end())
    {
      wsdebug(("> MailBoxMgr::deregisterOpenMailBox() removed!"));
      myOpenMailBoxes.erase(anIter);
    } // if
  } // if

  wsdebug(("< MailBoxMgr::deregisterOpenMailBox() MailBox: %p stream: %p"
         , theMailBox
         , theMailBox->stream()
         ));
} // MailBoxMgr::deregisterOpenMailBox()

//-------------------------------------------------------------------------
//     MailBoxMgr::instance()
//-------------------------------------------------------------------------
MailBoxMgr *MailBoxMgr::instance()
{
//  wsdebug(("> MailBoxMgr::instance() myInstance: %p", myInstance));
  if (myInstance == 0)
  {
    wsdebug(("> MailBoxMgr::instance() creating instance..."));
    myInstance = new MailBoxMgr();
    wsdebug(("< MailBoxMgr::instance() creating instance...done"));
  } // if

//  wsdebug(("< MailBoxMgr::instance() myInstance: %p", myInstance));
  return myInstance;
} // MailBoxMgr::instance()

//-------------------------------------------------------------------------
//     MailBoxMgr::terminate()
//-------------------------------------------------------------------------
void MailBoxMgr::terminate()
{
  wsdebug(("> MailBoxMgr::terminate() myInstance: %p", myInstance));

  if (myInstance)
  {
    //----- destroy MaiLBox-instances -----
    wsdebug(("  MailBoxMgr::terminate() calling destroyAll()..."));
    myInstance->destroyAll();

    //----- destroy myself ----------------
    wsdebug(("  MailBoxMgr::terminate() deleting myInstance..."));
    delete myInstance;
    myInstance = 0;
  } // if

  wsdebug(("< MailBoxMgr::terminate() myInstance: %p", myInstance));
} // MailBoxMgr::terminate()

//-------------------------------------------------------------------------
//     MailBoxMgr::destroyAll()
//-------------------------------------------------------------------------
void MailBoxMgr::destroyAll()
{
  wsdebug(("> MailBoxMgr::destroyAll() MailBoxCount: %d  OpenMailBoxes: %d"
         , myMailBoxes.size()
         , myOpenMailBoxes.size()
         ));

  MailBoxList::iterator anIter;
  while ((anIter = myMailBoxes.begin()) != myMailBoxes.end())
  {
    MailBox *aMailBox = *anIter;
    delete aMailBox;
  } // while

  wsdebug(("< MailBoxMgr::destroyAll()"));
} // MailBoxMgr::terminate()

//-------------------------------------------------------------------------
//     MailBoxMgr::getMailBox()
//-------------------------------------------------------------------------
MailBox *MailBoxMgr::getMailBox(MAILSTREAM *theStream)
{
  MailBox                    *aMailBox = 0;
  Stream2MailBoxMap::iterator anIter   = myOpenMailBoxes.find(theStream);

  if (anIter != myOpenMailBoxes.end())
  {
    aMailBox = anIter->second;
  } // if

  if (aMailBox == 0 && myActiveMailBox != 0)
  {
    aMailBox = myActiveMailBox;
  } // if

  return aMailBox;
} // MailBoxMgr::getMailBox()

//-------------------------------------------------------------------------
//     MailBoxMgr::getMailBoxCltn()
//-------------------------------------------------------------------------
const MailBoxList &MailBoxMgr::getMailBoxCltn() const
{
  return myMailBoxes;
} // MailBoxMgr::getMailBoxCltn()

//-------------------------------------------------------------------------
//     MailBoxMgr::hasMailBoxPtr()
//-------------------------------------------------------------------------
bool MailBoxMgr::hasMailBoxPtr(MailBox *theMailBox) const
{
  return std::find(myMailBoxes.begin(), myMailBoxes.end(), theMailBox) != myMailBoxes.end();
} // MailBoxMgr::hasMailBoxPtr()
