//-----------------------------------------------------------------------------
/*
 * Description: This file contains the class Ax::Mail::MailPart
 *
 * See the file README in the main directory for a description of
 * this software, copyright information, and how to reach the author.
 *
 * Author: alex
 * Date:   15.11.2003
 *
 * Last modfied:
 *   $Author: alex $
 *   $Date: 2004/01/17 14:37:44 $
 *   $Revision: 1.9 $
 */
//-----------------------------------------------------------------------------

//=============================================================================
//     includes
//=============================================================================
//----- qt --------------------------------------------------------------------
//----- CORBA -----------------------------------------------------------------
//----- ICE -------------------------------------------------------------------
//----- C++ -------------------------------------------------------------------
#include <algorithm>

//----- C ---------------------------------------------------------------------
#include <ctype.h>

//----- AxLib -----------------------------------------------------------------
#include <Ax/Tools/Trace.h>
// #include <Ax/Tools/TraceOn.h>

#include <Ax/Mail/Mail.h>
#include <Ax/Mail/MailPart.h>
#include <Ax/Mail/MailBox.h>
#include <Ax/Mail/MailBoxMgr.h>

//----- vdr -------------------------------------------------------------------
//----- local plugin ----------------------------------------------------------
//----- locals ----------------------------------------------------------------

//=============================================================================
//     defines
//=============================================================================

//=============================================================================
//     using namespaces
//=============================================================================
using namespace std;
using namespace Ax::Mail;

//=============================================================================
//     class MailPart
//=============================================================================

//-------------------------------------------------------------------------
//     MailPart::MailPart()
//-------------------------------------------------------------------------
MailPart::MailPart( const Mail       *theMail
                  , const MailPart   *theParent
                  , BODY             *theBody
                  , unsigned long     thePartNum
                  )
          : myMail   (theMail   )
          , myParent (theParent )
          , myBody   (theBody   )
          , myPartNum(thePartNum)
{
  wsdebug(("> MailPart::MailPart() PartNum: %ld MultiPart: %d Parent: %p  SectName: %-5s  PartNameFull: %-5s"
         , thePartNum
         , isMultiPart()
         , theParent
         , getPartName().c_str()
         , getPartNameFull().c_str()
         ));
         
  //----- check for sub-parts and create them too -----
  if (isMultiPart())
  {
    //----- multipart message -----
    int j = 0;
    for (PART *aPart  = myBody->nested.part;
               aPart != 0; 
               aPart  = aPart->next)
    {
      mySubParts.push_back(new MailPart(myMail, this, &aPart->body, ++j));
    } // for
  }
  else if ( (myBody->type == TYPEMESSAGE)
         && !strcmp (myBody->subtype, "RFC822")
         && (myBody->nested.msg->body != 0)
          )
  {
    //----- encapsulated message? -----
    BODY *aBody = myBody->nested.msg->body;

    mySubParts.push_back( new MailPart( myMail
                                      , this
                                      , aBody
                                      , (aBody->type == TYPEMULTIPART) ? 0 : 1
                                      )
                        );
  } // if
  
  
  wsdebug(("< MailPart::MailPart() PartNum: %ld MultiPart: %d Parent: %p  SectName: %-5s  PartNameFull: %-5s  SubParts: %d"
         , thePartNum
         , isMultiPart()
         , theParent
         , getPartName().c_str()
         , getPartNameFull().c_str()
         , mySubParts.size()
         ));
} // MailPart::MailPart()

//-------------------------------------------------------------------------
//     MailPart::~MailPart()
//-------------------------------------------------------------------------
MailPart::~MailPart()
{
  wsdebug(("< MailPart::~MailPart() PartNum: %ld MultiPart: %d Parent: %p  SectName: %-5s  PartNameFull: %-5s  SubParts: %d"
         , myPartNum
         , isMultiPart()
         , myParent
         , getPartName().c_str()
         , getPartNameFull().c_str()
         , mySubParts.size()
         ));
  for (MailPartVector::iterator anIter  = mySubParts.begin();
                                anIter != mySubParts.end();
                              ++anIter)
  {
    delete *anIter;
    *anIter = 0;
  } // while
  mySubParts.clear();

  wsdebug(("< MailPart::~MailPart()"));
} // MailPart::~MailPart()

//-------------------------------------------------------------------------
//     MailPart::getMail()
//-------------------------------------------------------------------------
const Mail *MailPart::getMail() const 
{ 
  return myMail;
} // MailPart::getMail()

//-------------------------------------------------------------------------
//     MailPart::getParent()
//-------------------------------------------------------------------------
const MailPart *MailPart::getParent() const
{
  return myParent;
} // MailPart::getParent()

//-------------------------------------------------------------------------
//     MailPart::getSubParts()
//-------------------------------------------------------------------------
const MailPartVector &MailPart::getSubParts() const
{
  return mySubParts;
} // MailPart::getSubParts()

//-------------------------------------------------------------------------
//     MailPart::getSubPart()
//-------------------------------------------------------------------------
const MailPart *MailPart::getSubPart(unsigned long thePartNum) const
{
  return (1 <= thePartNum && thePartNum <= mySubParts.size())
         ? mySubParts[thePartNum - 1]
         : 0;
} // MailPart::getSubPart()

//-------------------------------------------------------------------------
//     MailPart::getSubPart()
//-------------------------------------------------------------------------
unsigned long MailPart::getPartNum() const
{
  return myPartNum;
} // MailPart::getPartNum()

//-------------------------------------------------------------------------
//     MailPart::getType()
//-------------------------------------------------------------------------
unsigned short MailPart::getType() const
{
  return myBody->type;
} // MailPart::getType()

//-------------------------------------------------------------------------
//     MailPart::getEnc()
//-------------------------------------------------------------------------
unsigned short MailPart::getEnc() const
{
  return myBody->encoding;
} // MailPart::getEnc()

//-------------------------------------------------------------------------
//     MailPart::getCountLines()
//-------------------------------------------------------------------------
unsigned long MailPart::getCountLines() const
{ 
  return myBody->size.lines; 
} // MailPart::getCountLines()

//-------------------------------------------------------------------------
//     MailPart::getCountBytes()
//-------------------------------------------------------------------------
unsigned long MailPart::getCountBytes() const
{ 
  return myBody->size.bytes; 
} // MailPart::getCountBytes()

//-------------------------------------------------------------------------
//     MailPart::getTypeStr()
//-------------------------------------------------------------------------
std::string MailPart::getTypeStr() const 
{ 
  return body_types[myBody->type]; 
} // MailPart::getTypeStr()

//-------------------------------------------------------------------------
//     MailPart::getEncStr()
//-------------------------------------------------------------------------
std::string MailPart::getEncStr() const 
{ 
  return body_encodings[myBody->encoding]; 
} // MailPart::getEncStr()

//-------------------------------------------------------------------------
//     MailPart::getSubType()
//-------------------------------------------------------------------------
std::string MailPart::getSubType() const 
{ 
  return myBody->subtype ? myBody->subtype : "" ;
} // MailPart::getSubType()

//-------------------------------------------------------------------------
//     MailPart::getID()
//-------------------------------------------------------------------------
std::string MailPart::getID() const 
{ 
  return myBody->id ? myBody->id : "" ;
} // MailPart::getID()

//-------------------------------------------------------------------------
//     MailPart::getDesc()
//-------------------------------------------------------------------------
std::string MailPart::getDesc() const 
{ 
  return myBody->description ? myBody->description : "" ;
} // getDesc()

//-------------------------------------------------------------------------
//     MailPart::getMD5()
//-------------------------------------------------------------------------
std::string MailPart::getMD5() const 
{ 
  return myBody->md5 ? myBody->md5 : "" ;
} // MailPart::getMD5()

//-------------------------------------------------------------------------
//     MailPart::getPartName()
//-------------------------------------------------------------------------
std::string MailPart::getPartName() const
{
  std::string aName;
  
  if (!isMultiPart())
  {
    aName = getPartNameFull();
  } // if
  
  return aName;
} // MailPart::getPartName()

//-------------------------------------------------------------------------
//     MailPart::getPartNameFull()
//-------------------------------------------------------------------------
std::string MailPart::getPartNameFull() const
{
  std::string aName;

  if (isMultiPart())
  {
    if (getParent() != 0)
    {
      aName = getParent()->getPartNameFull();
    }
    else
    {
      aName = "";
    }
  } // if
  else // !isMultiPart()
  {
    std::string aParentName = getParent() ? getParent()->getPartNameFull() : "";
    if (aParentName != "")
    {
      char szText[20];
      sprintf(szText, ".%ld", getPartNum());
      aName = aParentName + std::string(szText);
    }
    else
    {
      char szText[20];
      sprintf(szText, "%ld", getPartNum());
      aName = std::string(szText);
    } // if
  } // if
  
  return aName;
} // MailPart::getPartNameFull()

//-------------------------------------------------------------------------
//     MailPart::isMultiPart()
//-------------------------------------------------------------------------
bool MailPart::isMultiPart() const
{
  return (myBody->type == TYPEMULTIPART);
} // MailPart::isMultiPart()
    
//-------------------------------------------------------------------------
//     MailPart::getParams()
//-------------------------------------------------------------------------
String2StringMap MailPart::getParams() const
{
  String2StringMap aMap;
  PARAMETER       *aPar = myBody->parameter;

  if (aPar != 0)
  {
    do
    {
      aMap[aPar->attribute] = aPar->value ? aPar->value : "";
    }  while ((aPar = aPar->next) != 0);
  } // if

  return aMap;
} // MailPart::getParams()

//-------------------------------------------------------------------------
//     MailPart::getParam()
//-------------------------------------------------------------------------
std::string MailPart::getParam(std::string theParam, bool *theFFound) const
{
  bool        fFound = false;
  std::string aValue("");
  PARAMETER  *aPar   = myBody->parameter;

  if (aPar != 0 && theParam.length() > 0)
  {
    std::transform(theParam.begin(), theParam.end(), theParam.begin(), toupper);

    do
    {
      std::string aParam(aPar->attribute);
      std::transform(aParam.begin(), aParam.end(), aParam.begin(), toupper);

      fFound = (aParam == theParam);
      if (fFound)
      {
        aValue = aPar->value ? std::string(aPar->value) : std::string("");
        std::transform(aValue.begin(), aValue.end(), aValue.begin(), toupper);
      } // if
    } while ((aPar = aPar->next) != 0 && !fFound);
  } // if

  if (theFFound)
  {
    *theFFound = fFound;
  } // if

  return aValue;
} // MailPart::getParam()

//-------------------------------------------------------------------------
//     MailPart::addTo()
//-------------------------------------------------------------------------
void MailPart::addTo(MailPartList &theList, bool fWithMultiPart) const
{
  if (!isMultiPart() || fWithMultiPart)
  {
    theList.push_back(this);
  } // if
  for (MailPartVector::const_iterator anIter  = mySubParts.begin();
                                        anIter != mySubParts.end();
                                      ++anIter)
  {
    (*anIter)->addTo(theList, fWithMultiPart);
  } // for
} // MailPart::addTo()

//-------------------------------------------------------------------------
//     MailPart::getBodyText()
//-------------------------------------------------------------------------
std::string MailPart::getBodyText() const
{
  unsigned long aLen = 0;
  long          aFlags = (!myMail->getMailBox()->getSetup()->fAutoMarkSeen) ? FT_PEEK : 0;

  wsinfo(("retrievingpart %s of mail %ld in mailbox '%s'..."
        , getPartName().c_str(), myMail->getMailNum(), myMail->getMailBox()->getMailBox()
        ));

  MailBoxMgr::instance()->setActiveMailBox(myMail->getMailBox());
  char *aBody = mail_fetchbody_full( myMail->getMailBox()->stream()
                                   , myMail->getMailNum()
                                   , const_cast<char *>(getPartName().c_str())
                                   , &aLen
                                   , aFlags
                                   );
  MailBoxMgr::instance()->setActiveMailBox(0);

  std::string aStr;

  if (aBody == 0)
  {
    wserror(("retrievingpart %s of mail %ld in mailbox '%s'...FAILED"
          , getPartName().c_str(), myMail->getMailNum(), myMail->getMailBox()->getMailBox()
          ));
  }
  else if (myBody && aLen == myBody->size.bytes)
  {
    char *aDecoded = 0;

    if (myBody->encoding == ENCQUOTEDPRINTABLE)
    {
      unsigned long aDecLen = 0;

      aDecoded = (char *)rfc822_qprint((unsigned char *)aBody, aLen, &aDecLen);

      if (aDecoded)
      {
        aStr = aDecoded;
        fs_give((void **)&aDecoded);

        wsinfo(("decoded 'quoted-printable' of part %s in mail %ld in mailbox '%s'"
              , getPartName().c_str(), myMail->getMailNum(), myMail->getMailBox()->getMailBox()
              ));
      }
      else
      {
        wswarn(("decoding of 'quoted-printable' failed for part %s in mail %ld in mailbox '%s'"
              , getPartName().c_str(), myMail->getMailNum(), myMail->getMailBox()->getMailBox()
              ));

        aStr = aBody;
      } // if
    }
    else
    {
      wsinfo(("retrievingpart %s of mail %ld in mailbox '%s'...OK"
            , getPartName().c_str(), myMail->getMailNum(), myMail->getMailBox()->getMailBox()
            ));
      aStr = aBody;
    } // if
  } // if

  return aStr;
} // MailPart::getBodyText()
