//-----------------------------------------------------------------------------
/*
 * Description: This file contains the class AxMailChecker
 *
 * See the file README in the main directory for a description of
 * this software, copyright information, and how to reach the author.
 *
 * Author: alex
 * Date:   26.12.2003
 *
 * Last modfied:
 *   $Author: alex $
 *   $Date: 2004/01/10 21:05:45 $
 *   $Revision: 1.1 $
 */
//-----------------------------------------------------------------------------
#ifndef __AxMailChecker_H__
#define __AxMailChecker_H__

//-----------------------------------------------------------------------------
//     includes
//-----------------------------------------------------------------------------

//----- qt --------------------------------------------------------------------
//----- CORBA -----------------------------------------------------------------
//----- C++ -------------------------------------------------------------------
//----- C ---------------------------------------------------------------------
//----- AxLib -----------------------------------------------------------------
#include <Ax/Mail/Globals.h>

//----- local -----------------------------------------------------------------
//----- vdr -------------------------------------------------------------------
#include <vdr/thread.h>

//----- local -----------------------------------------------------------------

//=============================================================================
//     forward declarations
//=============================================================================
class AxPluginSettings;
namespace Ax
{
  namespace Mail
  {
    class MailBox;
  };
};

//=============================================================================
//     typedefs
//=============================================================================

//=============================================================================
//     class AxMailChecker
//=============================================================================
class AxMailChecker : public cThread
{
  public:
    //-------------------------------------------------------------------------
    //     AxMailChecker()
    //-------------------------------------------------------------------------
    AxMailChecker(const AxPluginSettings *thePluginSettings);

    //-------------------------------------------------------------------------
    //     ~AxMailChecker()
    //-------------------------------------------------------------------------
    ~AxMailChecker();

    //-------------------------------------------------------------------------
    //     startChecking()
    //-------------------------------------------------------------------------
    bool startChecking();

    //-------------------------------------------------------------------------
    //     stopChecking()
    //-------------------------------------------------------------------------
    void stopChecking();

  protected:
    //-------------------------------------------------------------------------
    //     Action()
    //-------------------------------------------------------------------------
    virtual void Action();

    //-------------------------------------------------------------------------
    //     checkMailBox()
    //-------------------------------------------------------------------------
    /** request the mutex and check one mailbox.
     */
    void checkMailBox(Ax::Mail::MailBox *theMB, bool fForceStartExternal);

  public:
    //-------------------------------------------------------------------------
    //     attributes
    //-------------------------------------------------------------------------
    /** Plugin settings
     */
    const AxPluginSettings *myPluginSettings;

    /** Stop the thread
     */
    bool   fExit;

    /** Mutex is locked during a mail-check.
     *  So if main thread want's to set fExit, it first requests the mutex
     *  (which gives the thread the chance to finish the current mailbox)
     *  and sets fExit afterwards
     */
    cMutex myCheckMutex;

}; // class AxSettings

#endif

