//-----------------------------------------------------------------------------
/*
 * Description: This file contains the class AxMailTools
 *
 * See the file README in the main directory for a description of
 * this software, copyright information, and how to reach the author.
 *
 * Author: alex
 * Date:   12.12.2003
 *
 * Last modfied:
 *   $Author: alex $
 *   $Date: 2003/12/19 17:00:07 $
 *   $Revision: 1.2 $
 */
//-----------------------------------------------------------------------------
#ifndef __AxMailTools_H__
#define __AxMailTools_H__

//-----------------------------------------------------------------------------
//     includes
//-----------------------------------------------------------------------------

//----- qt --------------------------------------------------------------------
//----- CORBA -----------------------------------------------------------------
//----- C++ -------------------------------------------------------------------
#include <string>
#include <map>

//----- C ---------------------------------------------------------------------
//----- AxLib -----------------------------------------------------------------
//----- local -----------------------------------------------------------------

//=============================================================================
//     class AxMailTools
//=============================================================================
/** The class AxMailTools contains some helper functions for the mailbox-plugin.
 */
class AxMailTools
{
  private:
    //-----------------------------------------------------------------------
    //     tables for character mappings
    //-----------------------------------------------------------------------
    static const short         Map_Base64[];
    static const unsigned char Map_ISO_8859_1_to_VDR[];
    static const unsigned char Map_ISO_8859_X_to_VDR[];
    static const unsigned char Map_ISO_8859_15_to_VDR[];
    static const unsigned char Map_Windows_1252_to_VDR[];

  public:
    //-----------------------------------------------------------------------
    //     typedefs
    //-----------------------------------------------------------------------
    typedef std::map<std::string, int> AxEncStatMap;

    //-----------------------------------------------------------------------
    //     class AxEncStat
    //-----------------------------------------------------------------------
    /** To collect statistics
     */
     class AxEncStat
     {
       public:
         AxEncStat()
           : CountCodesOK (0)
           , CountCodesNOK(0)
         { }

         int          CountCodesOK;
         int          CountCodesNOK;
         AxEncStatMap MapEnc ;
     }; // class AxEncStat


    //-----------------------------------------------------------------------
    //     decodeEncodedWords()
    //-----------------------------------------------------------------------
    static bool decodeEncodedWords( const std::string  &theSrc
                                  ,       std::string  &theDst
                                  ,       AxEncStat    *theStat = 0
                                  );

  private:
    //-----------------------------------------------------------------------
    //     decodeEncodedWord()
    //-----------------------------------------------------------------------
    static bool decodeEncodedWord( const std::string  &theCharSet
                                 , const std::string  &theEncoding
                                 , const std::string  &theSrc
                                 ,       std::string  &theDst
                                 ,       AxEncStat    *theStat = 0
                                 );

    //-----------------------------------------------------------------------
    //     decodeQuoted()
    //-----------------------------------------------------------------------
    static bool decodeQuoted( const unsigned char *theTable
                            , const std::string   &theSrc
                            ,       std::string   &theDst
                            );

    //-----------------------------------------------------------------------
    //     decodeBase64()
    //-----------------------------------------------------------------------
    static bool decodeBase64( const unsigned char *theTable
                            , const std::string   &theSrc
                            ,       std::string   &theDst
                            );

}; // namespace AxMailTools

#endif // __AxMailTools_H__
