//-----------------------------------------------------------------------------
/*
 * Description: This file contains the class AxAxMenuCode
 *
 * See the file README in the main directory for a description of
 * this software, copyright information, and how to reach the author.
 *
 * Author: alex
 * Date:   15.11.2003
 *
 * Last modfied:
 *   $Author: alex $
 *   $Date: 2004/05/31 11:36:52 $
 *   $Revision: 1.3 $
 */
//-----------------------------------------------------------------------------

//=============================================================================
//     includes
//=============================================================================

//----- multiple defs of swap() workaround ------------------------------------
#include "AxPluginGlobals.h"

//----- qt --------------------------------------------------------------------
//----- CORBA -----------------------------------------------------------------
//----- ICE -------------------------------------------------------------------
//----- C++ -------------------------------------------------------------------
#include <algorithm>

//----- C ---------------------------------------------------------------------
//----- vdr -------------------------------------------------------------------
#include <vdr/i18n.h>
#include <vdr/menuitems.h>

//----- AxLib -----------------------------------------------------------------
#include <Ax/Tools/Trace.h>
// #include <Ax/Tools/TraceOn.h>

//----- local plugin ----------------------------------------------------------
#include "AxMenuCode.h"
#include "AxVdrTools.h"

//=============================================================================
//     using namespaces
//=============================================================================
using namespace std;

//=============================================================================
//     globals
//=============================================================================

//----- locals ----------------------------------------------------------------

//-----------------------------------------------------------------------------
//     AxMenuCode::AxMenuCode()
//-----------------------------------------------------------------------------
AxMenuCode::AxMenuCode(const char *theCode, int theCodeMaxLen, bool &fCodeOK)
            : PARENT(tr("Restricted area"), strlen(tr("Access code")) + 1)
            , myCodeMaxLen(theCodeMaxLen)
            , myCodeOk    (fCodeOK)
            , myReqCode   (0)
            , myActCode   (0)
            , myCodeItem  (0)
{
  wsdebug(("> AxMenuCode::AxMenuCode() MaxLen: %d  Code: '%s'", myCodeMaxLen, theCode));

  //----- init attributes -------------------------------------------------
  myReqCode = (char *)calloc(myCodeMaxLen + 1, sizeof(char));
  myActCode = (char *)calloc(myCodeMaxLen + 1, sizeof(char));
  strncpy(myReqCode, theCode, myCodeMaxLen);

  myCodeOk = false;

  //----- create some empty lines------------------------------------------
  for (int i = 1; i <= std::max(DisplayMenu()->MaxItems() / 2 - 1, 0); ++i)
  {
    cOsdItem *anItem = new cOsdItem("");
    anItem->SetSelectable(false);
    Add(anItem);
  } // for

  //----- create the access-code item -------------------------------------
  myCodeItem = new cMenuEditNumItem(tr("Access code"), myActCode, myCodeMaxLen, true);
  Add(myCodeItem);
  SetCurrent(myCodeItem);

  wsdebug(("< AxMenuCode::AxMenuCode() Count: %d", Count()));
} // AxMenuCode::AxMenuCode()

//-----------------------------------------------------------------------------
//     AxMenuCode::~AxMenuCode()
//-----------------------------------------------------------------------------
AxMenuCode::~AxMenuCode()
{
  wsdebug(("> AxMenuCode::~AxMenuCode() CodeOK: %d  ActCode: '%s'  ReqCode: '%s'"
         , myCodeOk, myActCode, myReqCode
         ));
  free(myActCode);
  free(myReqCode);
  wsdebug(("< AxMenuCode::~AxMenuCode()"));
} // AxMenuCode::~AxMenuCode()

#define DEBUG_PROCESS_KEY 0

//-------------------------------------------------------------------------
//     AxMenuCode::ProcessKey()
//-------------------------------------------------------------------------
eOSState AxMenuCode::ProcessKey(eKeys theKey)
{
#if DEBUG_PROCESS_KEY
  if (theKey != kNone)
  {
    wsdebug(("> AxMenuCode::ProcessKey()      Key: %-10s  Cur: %3d  HasSub: %d  parent processing..."
          , cKey::ToString(theKey)
          , Current()
          , HasSubMenu()
          ));
  } // if
#endif

  bool fHasSubMenu = HasSubMenu();
  eOSState aState = PARENT::ProcessKey(theKey);

#if DEBUG_PROCESS_KEY
  if (theKey != kNone)
  {
    wsdebug(("  AxMenuCode::ProcessKey()      Key: %-10s  Cur: %3d  HasSub: %d  State: %-10s  "
          , cKey::ToString(theKey)
          , Current()
          , HasSubMenu()
          , AxVdrTools::getOSStateStr(aState)
          ));
  } // if
#endif

  if (aState != osUnknown && fHasSubMenu)
  {
#if DEBUG_PROCESS_KEY
    if (theKey != kNone)
    {
      wsdebug(("< AxMenuCode::ProcessKey()      Key: %-10s  Cur: %3d  HasSub: %d  State: %-10s  already handled"
            , cKey::ToString(theKey)
            , Current()
            , HasSubMenu()
            , AxVdrTools::getOSStateStr(aState)
            ));
    } // if
#endif
    return aState;
  } // if

  //----- select action... ----------------------------------------------------
  switch (theKey)
  {
    //----- check the code and quit if ok or display error msg ----
    case kOk:
    {
      myCodeOk = strcmp(myActCode, myReqCode) == 0;

      wsdebug(("  AxMenuCode::ProcessKey() CodeOK: %d  ReqCode: '%s'  ActCode: '%s'"
             , myCodeOk, myReqCode, myActCode
             ));

      if (myCodeOk)
      {
        aState = osBack;
      }
      else
      {
        memset(myActCode, 0, sizeof(myActCode));
        Skins.Message(mtStatus, tr("Code invalid!"), 3);
        aState = osContinue;
      } // if
      break;
    }

    //----- check the code immediately and quit if ok -----
    case k0:
    case k1:
    case k2:
    case k3:
    case k4:
    case k5:
    case k6:
    case k7:
    case k8:
    case k9:
    {
//      Skins.Message(mtError, "");
      // immediately close MenuCode if code is valid (without pressing the kOk)
      myCodeOk = strcmp(myActCode, myReqCode) == 0;
      wsdebug(("  AxMenuCode::ProcessKey() CodeOK: %d  ReqCode: '%s'  ActCode: '%s'"
             , myCodeOk, myReqCode, myActCode
             ));

      if (myCodeOk)
      {
        aState = osBack;
      }
      myCodeItem->Set();
      Display();
      break;
    }

    default:
    {
      break;
    }
  } // switch

#if DEBUG_PROCESS_KEY
  if (theKey != kNone)
  {
    wsdebug(("< AxMenuCode::ProcessKey()      Key: %-10s  Cur: %3d  HasSub: %d  State: %-10s  "
          , cKey::ToString(theKey)
          , Current()
          , HasSubMenu()
          , AxVdrTools::getOSStateStr(aState)
          ));
  } // if
#endif
  return aState;
} // AxMenuCode::ProcessKey()
