//-----------------------------------------------------------------------------
/*
 * Description: This file contains the class AxMenuMailView
 *
 * See the file README in the main directory for a description of
 * this software, copyright information, and how to reach the author.
 *
 * Author: alex
 * Date:   15.11.2003
 *
 * Last modfied:
 *   $Author: alex $
 *   $Date: 2004/05/31 12:52:20 $
 *   $Revision: 1.5 $
 */
//-----------------------------------------------------------------------------
#ifndef __AxMenuMailView_H__
#define __AxMenuMailView_H__

//=============================================================================
//     includes
//=============================================================================

//----- qt --------------------------------------------------------------------
//----- CORBA -----------------------------------------------------------------
//----- C++ -------------------------------------------------------------------
#include <string>

//----- C ---------------------------------------------------------------------
//----- vdr -------------------------------------------------------------------
#include <vdr/osd.h>
#include <vdr/menuitems.h>

//----- AxLib -----------------------------------------------------------------
#include <Ax/Mail/Globals.h>

//----- local -----------------------------------------------------------------

//=============================================================================
//     forward declarations
//=============================================================================
class AxMenuMailListView;
class AxPluginMailBox;
class AxOsdItem;
#if ALEX_NOPORT
class AxOsdMailItem;
#endif

//=============================================================================
//     class AxMenuMailView
//=============================================================================
/** Menu to display one mail.
 */
class AxMenuMailView : public cOsdMenu
{
  typedef cOsdMenu PARENT;

    //-------------------------------------------------------------------------
    //     enum AxPage
    //-------------------------------------------------------------------------
    enum AxPage
    {
      PAGE_JUMP
    , PAGE_MARK
    }; // enum AxPage

  public:
    //-------------------------------------------------------------------------
    //     AxMenuMailView()
    //-------------------------------------------------------------------------
    /** Constructor
     */
    AxMenuMailView( AxPluginMailBox    *thePlugin
                  , Ax::Mail::MailBox  *theMailBox
                  , Ax::Mail::Mail     *theMail
                  , AxMenuMailListView *theMailListView
                  );

    //-------------------------------------------------------------------------
    //     ~AxMenuMailView()
    //-------------------------------------------------------------------------
    /** Destructor
     */
    virtual ~AxMenuMailView();

    //-------------------------------------------------------------------------
    //     ProcessKey()
    //-------------------------------------------------------------------------
    /** Process key press events
     */
    virtual eOSState ProcessKey(eKeys Key);

    //-------------------------------------------------------------------------
    //     Display()
    //-------------------------------------------------------------------------
    virtual void Display(void);

  protected:
    //-------------------------------------------------------------------------
    //     showMail()
    //-------------------------------------------------------------------------
    /** update the menuitems to display the current mail
     */
    void showMail();

    //-------------------------------------------------------------------------
    //     setTitle()
    //-------------------------------------------------------------------------
    /** update the title bar
     */
    void setTitle();

    //-------------------------------------------------------------------------
    //     setButtons()
    //-------------------------------------------------------------------------
    /** update the buttons
     */
    void setButtons(AxPage thePage);

  private:
    //-------------------------------------------------------------------------
    //     attributes
    //-------------------------------------------------------------------------
    AxPluginMailBox    *myPlugin ; ///< pointer to the plugin
    Ax::Mail::MailBox  *myMailBox; ///< pointer to the open mailbox
    Ax::Mail::Mail     *myMail   ; ///< the mail which is displayed
    Ax::Mail::Mail     *myPrevNew; ///< cache the prev new mail, updated in showMail()
    Ax::Mail::Mail     *myNextNew; ///< cache the next new mail, updated in showMail()
    Ax::Mail::Mail     *myPrev   ; ///< cache the prev mail    , updated in showMail()
    Ax::Mail::Mail     *myNext   ; ///< cache the next mail    , updated in showMail()

    AxMenuMailListView *myMailListView; ///< the 'parent'-view from which this was opened
    AxPage              myButtonPage;   ///< page identifier

#if ALEX_NOPORT
    AxOsdItem          *mySubjectItem;
    AxOsdItem          *mySenderItem;
    AxOsdItem          *myDateItem;
    AxOsdMailItem      *myBodyItem;
#else
    std::string         myBodyText;
#endif

}; // class AxMenuMailView

#endif

