//-----------------------------------------------------------------------------
/*
 * Description: This file contains the class AxOsdMailItem
 *
 * See the file README in the main directory for a description of
 * this software, copyright information, and how to reach the author.
 *
 * Author: alex
 * Date:   17.11.2003
 *
 * Last modfied:
 *   $Author: alex $
 *   $Date: 2004/01/23 12:18:35 $
 *   $Revision: 1.5 $
 */
//-----------------------------------------------------------------------------
#ifndef __AxOsdMailItem_H__
#define __AxOsdMailItem_H__

//-----------------------------------------------------------------------------
//     includes
//-----------------------------------------------------------------------------

//----- qt --------------------------------------------------------------------
//----- CORBA -----------------------------------------------------------------
//----- C++ -------------------------------------------------------------------
#include <string>
#include <vector>

//----- C ---------------------------------------------------------------------
//----- AxLib -----------------------------------------------------------------
//----- VDR -------------------------------------------------------------------
#include <vdr/menuitems.h>
#include <vdr/status.h>

//----- local -----------------------------------------------------------------

//=============================================================================
//     forward declarations
//=============================================================================
namespace Ax
{
  namespace Mail
  {
    class Mail;
    class MailPart;
  };
};

//=============================================================================
//     class AxOsdMailItem
//=============================================================================
/** This class is used to display the mail text.
 *
 * The source of this class is based on the source of VDRs cMenuTextItem.
 */
class AxOsdMailItem : public cOsdItem
{
  typedef cOsdItem PARENT;

  public:
    //-------------------------------------------------------------------------
    //     enums
    //-------------------------------------------------------------------------
    enum MailViewMode
    {
      MVM_FIRST = 0

    , MVM_HARD    = MVM_FIRST ///< 'hard' word wrap
    , MVM_QUOTE   = 1         ///< continue quotes
    , MVM_FLOWED  = 2         ///< honor 'format=flowed` (RFC 2646)
    , MVM_SMALL   = 3         ///< continue lines which end and start with small chars

    , MVM_LAST    = MVM_SMALL
    };

    //-------------------------------------------------------------------------
    //     typedefs
    //-------------------------------------------------------------------------
    struct AxLine
    {
      AxLine(const std::string &theText = std::string(""), bool theFSep = false)
        : Text(theText)
        , fSep(theFSep)
      {
      }

      void clear()
      {
        Text = std::string("");
        fSep = false;
      }

      std::string Text;
      bool        fSep;
    };

    typedef std::vector<AxLine> LineCltn;

    //-------------------------------------------------------------------------
    //     AxOsdMailItem()
    //-------------------------------------------------------------------------
    AxOsdMailItem( int X
                 , int Y
                 , int W
                 , int H
                 , eDvbColor theFgColor  = clrWhite
                 , eDvbColor theBgColor  = clrBackground
                 , eDvbFont  theFont     = fontOsd
                 , int       theJumpSize = 1
                 );

    //-------------------------------------------------------------------------
    //     ~AxOsdMailItem()
    //-------------------------------------------------------------------------
    ~AxOsdMailItem();

    //-------------------------------------------------------------------------
    //     Height()
    //-------------------------------------------------------------------------
    int Height(void);

    //-------------------------------------------------------------------------
    //     Clear()
    //-------------------------------------------------------------------------
    void Clear(void);

    //-------------------------------------------------------------------------
    //     Display()
    //-------------------------------------------------------------------------
    /** overwritten from cOsdItem
     */
    virtual void Display(int Offset = -1, eDvbColor FgColor = clrWhite, eDvbColor BgColor = clrBackground);

    //-------------------------------------------------------------------------
    //     ProcessKey()
    //-------------------------------------------------------------------------
    /** overwritten from cOsdItem
     */
    virtual eOSState ProcessKey(eKeys Key);

    //-------------------------------------------------------------------------
    //     setMail()
    //-------------------------------------------------------------------------
    void setMail(Ax::Mail::Mail *theMail);

    //-------------------------------------------------------------------------
    //     setViewMode()
    //-------------------------------------------------------------------------
    void setViewMode(MailViewMode theMode);

    //-------------------------------------------------------------------------
    //     getViewMode()
    //-------------------------------------------------------------------------
    MailViewMode getViewMode() const;

    //-------------------------------------------------------------------------
    //     getViewModeName()
    //-------------------------------------------------------------------------
    const char *getViewModeName() const;

  private:
    //-------------------------------------------------------------------------
    //     CanScrollUp()
    //-------------------------------------------------------------------------
    bool CanScrollUp(void);

    //-------------------------------------------------------------------------
    //     CanScrollDown()
    //-------------------------------------------------------------------------
    bool CanScrollDown(void);

    //-------------------------------------------------------------------------
    //     ScrollUp()
    //-------------------------------------------------------------------------
    void ScrollUp(bool Page);

    //-------------------------------------------------------------------------
    //     ScrollDown()
    //-------------------------------------------------------------------------
    void ScrollDown(bool Page);
  
    //-----------------------------------------------------------------------------
    //     getQuotes()
    //-----------------------------------------------------------------------------
    std::string getQuotes(const std::string &theStr);

    //-------------------------------------------------------------------------
    //     splitText()
    //-------------------------------------------------------------------------
    LineCltn splitText(const std::string &theStr);

    //-------------------------------------------------------------------------
    //     getFlowedText()
    //-------------------------------------------------------------------------
    LineCltn getFlowedText(const LineCltn &theSrc);

    //-------------------------------------------------------------------------
    //     improveWrap()
    //-------------------------------------------------------------------------
    LineCltn improveWrap(const LineCltn &theSrc);

    //-------------------------------------------------------------------------
    //     wrapText()
    //-------------------------------------------------------------------------
    LineCltn wrapText(const LineCltn &theSrc);

    //-------------------------------------------------------------------------
    //     getPartText()
    //-------------------------------------------------------------------------
    LineCltn getPartText(const Ax::Mail::MailPart *thePart);

  private:
    //-------------------------------------------------------------------------
    //     attributes
    //-------------------------------------------------------------------------
    Ax::Mail::Mail *myMail;
    LineCltn        myText;       ///< osd-wrapped text to display (owner is this class)
    MailViewMode    myMode;       ///< mail view mode
    int             myXPos;       ///< x-pos  of item in osd-units (characters?)
    int             myYPos;       ///< y-pos  of item in osd-units (characters?)
    int             myWidth;      ///< width  of item in osd-units (characters?)
    int             myHeight;     ///< heigth of item in osd-units (characters?)
    int             myOffset;     ///< current line number displayed at the top (offset >= 0 < (lines - h)
    int             myJumpSize;   ///< number of lines to scroll with cursor keys

    eDvbColor       fgColor;
    eDvbColor       bgColor;
    eDvbFont        myFont;


}; // class AxOsdMailItem


#endif // __AxOsdMailItem_H__
