//-----------------------------------------------------------------------------
/*
 * Description: This file contains the class AxVdrTools
 *
 * See the file README in the main directory for a description of
 * this software, copyright information, and how to reach the author.
 *
 * Author: alex
 * Date:   15.11.2003
 *
 * Last modfied:
 *   $Author: alex $
 *   $Date: 2004/04/04 15:53:38 $
 *   $Revision: 1.13 $
 */
//-----------------------------------------------------------------------------

//=============================================================================
//     includes
//=============================================================================

//----- multiple defs of swap() workaround ------------------------------------
#include "AxPluginGlobals.h"

//----- qt --------------------------------------------------------------------
//----- CORBA -----------------------------------------------------------------
//----- ICE -------------------------------------------------------------------
//----- C++ -------------------------------------------------------------------
//----- C ---------------------------------------------------------------------
//----- vdr -------------------------------------------------------------------
#include <vdr/tools.h>
#include <vdr/i18n.h>
#include <vdr/menuitems.h>

//----- AxLib -----------------------------------------------------------------
#include <Ax/Tools/Trace.h>
// #include <Ax/Tools/TraceOn.h>
#include <Ax/Mail/MailBoxSetup.h>

//----- local plugin ----------------------------------------------------------
#include "AxVdrTools.h"

//----- locals ----------------------------------------------------------------

//=============================================================================
//     using namespaces
//=============================================================================
using namespace std;

//=============================================================================
//     class AxVdrTools
//=============================================================================

//-------------------------------------------------------------------------
//     AxVdrTools::getOSStateStr()
//-------------------------------------------------------------------------
const char *AxVdrTools::getOSStateStr(eOSState theState)
{
  switch (theState)
  {
    case osUnknown   : return "osUnknown";
    case osContinue  : return "osContinue";
    case osSchedule  : return "osSchedule";
    case osChannels  : return "osChannels";
    case osTimers    : return "osTimers";
    case osRecordings: return "osRecordings";
    case osPlugin    : return "osPlugin";
    case osSetup     : return "osSetup";
    case osCommands  : return "osCommands";
    case osPause     : return "osPause";
    case osRecord    : return "osRecord";
    case osReplay    : return "osReplay";
    case osStopRecord: return "osStopRecord";
    case osStopReplay: return "osStopReplay";
    case osCancelEdit: return "osCancelEdit";
    case osSwitchDvb : return "osSwitchDvb";
    case osBack      : return "osBack";
    case osEnd       : return "osEnd";
    case os_User     : return "os_User";
    case osUser1     : return "osUser1";
    case osUser2     : return "osUser2";
    case osUser3     : return "osUser3";
    case osUser4     : return "osUser4";
    case osUser5     : return "osUser5";
    case osUser6     : return "osUser6";
    case osUser7     : return "osUser7";
    case osUser8     : return "osUser8";
    case osUser9     : return "osUser9";
    case osUser10    : return "osUser10";
    default: break;
  } // switch
  return "NO_ENUM";
} // AxVdrTools::getOSStateStr()

//-------------------------------------------------------------------------
//     Fonts
//-------------------------------------------------------------------------
const int   AxVdrTools::FontCount = 2;
const char *AxVdrTools::FontNames[AxVdrTools::FontCount];

//-------------------------------------------------------------------------
//     Access modes
//-------------------------------------------------------------------------
const int   AxVdrTools::AccessModeCount = 3;
const char *AxVdrTools::AccessModeNames[AxVdrTools::AccessModeCount];

//-------------------------------------------------------------------------
//     background-check modes
//-------------------------------------------------------------------------
#if VDR_HAS_ASYNC_MSG
const int   AxVdrTools::BGCheckModeCount = Ax::Mail::MailBoxSetup::BG_MODE_EXT_MSG + 1;
const char* AxVdrTools::BGCheckModeNames[AxVdrTools::BGCheckModeCount];
#else
const int   AxVdrTools::BGCheckModeCount = Ax::Mail::MailBoxSetup::BG_MODE_EXT     + 1;
const char* AxVdrTools::BGCheckModeNames[AxVdrTools::BGCheckModeCount];
#endif

//-------------------------------------------------------------------------
//     Sort order
//-------------------------------------------------------------------------
const char *AxVdrTools::SortOrderStrings[AxVdrTools::SO_COUNT];

//-------------------------------------------------------------------------
//     AxVdrTools::initialize()
//-------------------------------------------------------------------------
void AxVdrTools::initialize()
{
  //----- font names -----
  FontNames[0] = tr("normal");
  FontNames[1] = tr("small" );

  //----- access modes -----
  AccessModeNames[0] = tr("unrestricted");  // AM_NONE
  AccessModeNames[1] = tr("setup");         // AM_SETUP
  AccessModeNames[2] = tr("setup + read");  // AM_SETUP_READ

  //----- background-check modes -----
  BGCheckModeNames[Ax::Mail::MailBoxSetup::BG_MODE_NONE   ] = tr("off");
  BGCheckModeNames[Ax::Mail::MailBoxSetup::BG_MODE_EXT    ] = tr("ext. app.");

#if VDR_HAS_ASYNC_MSG
  BGCheckModeNames[Ax::Mail::MailBoxSetup::BG_MODE_MSG    ] = tr("osd-msg");
  BGCheckModeNames[Ax::Mail::MailBoxSetup::BG_MODE_EXT_MSG] = tr("ext. app. + osd-msg");
#endif

  //----- sort oder  -----
  SortOrderStrings[AxVdrTools::SO_ASC ] = tr("ascending" );
  SortOrderStrings[AxVdrTools::SO_DESC] = tr("descending");
} // AxVdrTools::initialize()

//-------------------------------------------------------------------------
//     AxVdrTools::getDvbFont()
//-------------------------------------------------------------------------
eDvbFont AxVdrTools::getDvbFont(int theFontNum)
{
  switch (theFontNum)
  {
#ifdef SMALL_FONT_ENUM
    case 1 : return SMALL_FONT_ENUM;
#endif
    default: break;
  } // switch
  return fontOsd;
} // AxVdrTools::getDvbFont()
